/**
 * 
 */
package com.up.sell.interceptor;

import java.lang.reflect.Field;
import java.text.DateFormat;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Properties;

import org.apache.ibatis.executor.Executor;
import org.apache.ibatis.mapping.BoundSql;
import org.apache.ibatis.mapping.MappedStatement;
import org.apache.ibatis.mapping.ParameterMapping;
import org.apache.ibatis.mapping.SqlCommandType;
import org.apache.ibatis.plugin.Interceptor;
import org.apache.ibatis.plugin.Intercepts;
import org.apache.ibatis.plugin.Invocation;
import org.apache.ibatis.plugin.Plugin;
import org.apache.ibatis.plugin.Signature;
import org.apache.ibatis.reflection.MetaObject;
import org.apache.ibatis.session.Configuration;
import org.apache.ibatis.session.ResultHandler;
import org.apache.ibatis.session.RowBounds;
import org.apache.ibatis.type.TypeHandlerRegistry;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import com.up.sell.util.ConfigurationUtils;
import com.up.sell.util.Constant;

/**
 * @author mingguangz
 * 
 */
@Component
@Intercepts({ @Signature(type = Executor.class, method = "update", args = { MappedStatement.class, Object.class }),
		@Signature(type = Executor.class, method = "query", args = { MappedStatement.class, Object.class, RowBounds.class, ResultHandler.class }) })
public class MybatisSQLInterceptor implements Interceptor {
	private static final Logger logger = LoggerFactory.getLogger(MybatisSQLInterceptor.class);
	@SuppressWarnings("unused")
	private Properties properties;

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.apache.ibatis.plugin.Interceptor#intercept(org.apache.ibatis.plugin
	 * .Invocation)
	 */
	public Object intercept(Invocation invocation) throws Throwable {
		MappedStatement mappedStatement = (MappedStatement) invocation.getArgs()[0];
		Object parameter = null;
		if (invocation.getArgs().length > 1) {
			parameter = invocation.getArgs()[1];
		}
		String sqlId = mappedStatement.getId();
		BoundSql boundSql = mappedStatement.getBoundSql(parameter);
		Configuration configuration = mappedStatement.getConfiguration();
		Object returnValue = null;
		long start = System.currentTimeMillis();
		/*if(ConfigurationUtils.getBooleanValue("defense_sql_injection")) {
			this.modifyParameter(invocation);//转移特殊字符，防止脚本注入
		}*/
		returnValue = invocation.proceed();
		long end = System.currentTimeMillis();
		long time = (end - start);
		if (time > ConfigurationUtils.getIntValue(Constant.SQL_TIME)) {
			String sql = getSql(configuration, boundSql, sqlId, time);
			logger.debug(sql);
		}
		return returnValue;
	}

	private String getSql(Configuration configuration, BoundSql boundSql, String sqlId, long time) {
		String sql = showSql(configuration, boundSql);
		StringBuilder str = new StringBuilder();
		str.append(sqlId);
		str.append("	");
		str.append(sql);
		str.append("	running time (millis) = ");
		str.append(time);
		return str.toString();
	}

	private String getParameterValue(Object obj) {
		String value = null;
		if (obj instanceof String) {
			value = "'" + obj.toString() + "'";
		} else if (obj instanceof Date) {
			DateFormat formatter = DateFormat.getDateTimeInstance(DateFormat.DEFAULT, DateFormat.DEFAULT, Locale.CHINA);
			value = "'" + formatter.format(obj) + "'";
		} else {
			if (obj != null) {
				value = obj.toString();
			} else {
				value = "";
			}

		}
		return value;
	}

	private String showSql(Configuration configuration, BoundSql boundSql) {
		Object parameterObject = boundSql.getParameterObject();
		List<ParameterMapping> parameterMappings = boundSql.getParameterMappings();
		String sql = boundSql.getSql().replaceAll("[\\s]+", " ");
		if (parameterMappings.size() > 0 && parameterObject != null) {
			TypeHandlerRegistry typeHandlerRegistry = configuration.getTypeHandlerRegistry();
			if (typeHandlerRegistry.hasTypeHandler(parameterObject.getClass())) {
				sql = sql.replaceFirst("\\?", getParameterValue(parameterObject));
			} else {
				MetaObject metaObject = configuration.newMetaObject(parameterObject);
				for (ParameterMapping parameterMapping : parameterMappings) {
					String propertyName = parameterMapping.getProperty();
					if (metaObject.hasGetter(propertyName)) {
						Object obj = metaObject.getValue(propertyName);
						sql = sql.replaceFirst("\\?", getParameterValue(obj));
					} else if (boundSql.hasAdditionalParameter(propertyName)) {
						Object obj = boundSql.getAdditionalParameter(propertyName);
						sql = sql.replaceFirst("\\?", getParameterValue(obj));
					}
				}
			}
		}
		return sql;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.apache.ibatis.plugin.Interceptor#plugin(java.lang.Object)
	 */
	public Object plugin(Object target) {
		return Plugin.wrap(target, this);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.apache.ibatis.plugin.Interceptor#setProperties(java.util.Properties)
	 */
	public void setProperties(Properties properties) {
		this.properties = properties;
	}
	private String cleanXSS(String value) {
        //You'll need to remove the spaces from the html entities below
		value = value.replaceAll("<", "&lt;").replaceAll(">", "&gt;");
		//value = value.replaceAll("\\(", "&#40;").replaceAll("\\)", "&#41;");
		value = value.replaceAll("'", "&#39;");
		value = value.replaceAll("eval\\((.*)\\)", "");
		value = value.replaceAll("[\\\"\\\'][\\s]*javascript:(.*)[\\\"\\\']", "\"\"");
//		value = value.replaceAll("script", "");
		return value;
	}
	
	@SuppressWarnings({ "unchecked", "rawtypes" })
	private void modifyParameter(Invocation invocation) {
		try {
			Object[] args = invocation.getArgs();
			for(int i=0;i<args.length;i++) {
				Object arg=args[i];
				if(arg instanceof MappedStatement) {
					MappedStatement ms = (MappedStatement)arg;
					SqlCommandType sqlCommandType = ms.getSqlCommandType();
					if(sqlCommandType == SqlCommandType.INSERT || sqlCommandType==SqlCommandType.UPDATE) {
						continue;
					} else {
						break;
					}
				}
				if (arg instanceof Map) {
					Map map=(Map)arg;
					for (Object key : map.keySet()) {
						map.put(key, this.modifyObject(map.get(key)));
					}
				} else {
					this.modifyObject(arg);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	private Object modifyObject(Object arg) throws IllegalArgumentException, IllegalAccessException {
		if(arg instanceof String) {
			return this.cleanXSS(arg.toString());
		}
		if(arg != null) {
			Field[] fields = arg.getClass().getDeclaredFields();
			for (int ii = 0; ii < fields.length; ii++) {
				fields[ii].setAccessible(true);
				Object value;
				value = fields[ii].get(arg);
				if (value instanceof String) {
					fields[ii]
							.set(arg, this.cleanXSS(value.toString()));
				}
			}
		}
		return arg;
	}

}
