/**
 * 
 */
package com.up.sell.task;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import com.up.sell.service.MemberOrderSerice;
import com.up.sell.service.MemberSerice;

/**
 * @author Administrator
 *定时任务类
 */
@Component
public class ScheduledTask {
	private static final Logger logger = LoggerFactory.getLogger(ScheduledTask.class);

	@Autowired
	private MemberOrderSerice memberOrderSerice;
	@Autowired
	private MemberSerice memberSerice;
	/**
	 * 24小时未支付订单自动取消,每个小时执行一次
	 * fixedDelay = 5000表示当前方法执行完毕5000ms后，Spring scheduling会再次调用该方法
	 */
	@Scheduled(fixedDelay = 1000*60*60)
    public void cancelOrder () {
        try {
			logger.info("24小时未支付订单自动取消");
			this.memberOrderSerice.cancelOrder(24);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
    }
	/**
	 * 每天零点重设每个会员剩余查看品牌联系人的次数
	 */
	@Scheduled(cron="0 0 0 * * ?")
    public void resetMemberViewCount () {
		try {
			logger.info("每天零点重设每个会员剩余查看品牌联系人的次数");
			this.memberSerice.updateViewCount(3);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
    }
	
	/**
	 * 每天零点查询会员是否过期
	 */
	@Scheduled(cron="0 0 0 * * ?")
    public void memberExpired () {
		try {
			logger.info("每天零点查询会员是否过期");
			this.memberSerice.memberExpired();
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
    }
}
